/******************************************************************************/
//
// Name: 	Programming S/PDIF on ADSP-21364 SHARC(R) DSP Processor
//
/******************************************************************************

(C) Copyright 2005 - Analog Devices, Inc.  All rights reserved.

File Name:		main.c

Date Modified:	21/03/05	-	JR	-	Rev 1.0

Software:       VisualDSP++3.5 (November update)

Purpose:		To demonstrate S/PDIF programming - Loop back TX-RX
				
*******************************************************************************/


/* 
This example code has the following set-up for the SDPIF TX and RX

1. SPORT 0 supplies I2S data to the SPDIF TX
2. The Bi-phase clock input to SPDIF TX is generated by PCGB
3. The Bit clock and Frame sync for SPORT 0 is generated by PCGA
4. SPORT0 is configured for external bit clock and external frame sync
5. The DIT from SPDIF TX is fed back to DIR pin of S/PDIF RX
6. SPDIF RX receives bi-phase input from DIT
7. SPORT 1 is configured to receive data from SPDIF RX OUT


*/

//Include header files

#include <stdio.h>
#include <Cdef21369.h>
#include <def21369.h>
#include <signal.h>

// Clock and frame sync dividors used by PCG
// 
#define CLKA_DIVIDER 16 //-- this provides serial bit clock to both S/PDIF TX and SPORT0
#define FSA_DIVIDER 1024//-- this provides Frame Sync to both S/PDIF TX and SPORT0
#define CLKB_DIVIDER 4  //-- this provides HFCLK to S/pDIF TX

//function declarations

//=======================================================================================
void PCG_A_B_OnDAI(void); //-- this is only to watch PCG generated signals on scope
void InitSRU(void); 	  //-- SRU configuration for PCGA,B, SPORT0 and SPDIF siganls
void PCG_A_B_Config(void);
void Sport0Init(void);
void Sport1Init(void);
void SPDIF_TX_Init(void);
void SPDIF_RX_Init(void);
void SPDIF_RxToSport1Sru(void);
void Sport0TxIsr(int dummy);
void Sport1RxIsr(int dummy);

//=======================================================================================

//Global definitions
//---------------------------------------------
int SPDIF_BlockBuffer[384];
int SubFrmCnt = 0;

int SPDIF_Rx_Buffer[384];
int RxSubFrmCnt = 0;
int TX_VALUE = 0x0;

int LeftChannel=1;
//---------------------------------------------

void main(void)
{
	//local variables
	int delay1;
	
	//Initialize SPDIF block in memory
	SPDIF_BlockBuffer[0] = 0x12345610; //Block start bit is set to "1"
	SPDIF_BlockBuffer[1] = 0x12345610; 
	
	for (SubFrmCnt = 2; SubFrmCnt <=383 ; SubFrmCnt++)
		{
			SPDIF_BlockBuffer[SubFrmCnt] = 0x12345600;
		}

	
	SubFrmCnt = 0;
	//----------------------------------------------------------------------------------	
	
	
	PCG_A_B_OnDAI();//--Connect the PCG signals to DAI pins to watch on scope
	InitSRU();
	SPDIF_RxToSport1Sru();
	PCG_A_B_Config();
	
	
	
	
	interrupts(SIG_P6,Sport0TxIsr);//set up sport0 interrupt
	interrupts(SIG_P3,Sport1RxIsr);//set up sport1 interrupt
	

	SPDIF_RX_Init();//Initialize SPDIF RX
	SPDIF_TX_Init();//Initialize SPDIF TX
	
	Sport0Init();//Initialize the SPORT0 for I2S mode	
	Sport1Init();//Initialize the SPORT1 for I2S mode	
	
	//Wait for SPORT interrupts
	while(1)
	{
	}
}


//Initialize SPORT0
void Sport0Init()
{
	*pSPCTL0 |= SPEN_A | SLEN32 | OPMODE | SPTRAN;		//SPORT0 configuration:
														//External clock and word select 
														//I2S mode
														//Transmitter
														//Word length 32
}

//Initialize SPORT1
void Sport1Init()
{
	*pSPCTL1 |= SPEN_A | SLEN32 | OPMODE;				//SPORT1 configuration:
														//External clock and word select 
														//I2S mode
														//Receiver
														//Word length 32
}
//Initialize SPDIF Transmitter
void SPDIF_TX_Init()
{
	*pDITCTL = DIT_EN | DIT_SMODEIN0;//configuration of SPDIF TX
}
//-------------------------------------------------------------

/* 
	SPORT 0 interrupt routine
   	The data from internal memory which is block data is trasmitted 
   	every time in the routine
   
*/   
void Sport0TxIsr(int dummy)
{
	*pTXSP0A = SPDIF_BlockBuffer[SubFrmCnt % 384];
	SubFrmCnt++;
}
//

void Sport1RxIsr(int dummy)
{
	
	SPDIF_Rx_Buffer[RxSubFrmCnt % 384] = *pRXSP1A;
	RxSubFrmCnt++;
}

//-------------------------------------------------------------
//PCG "A" and PCG "B" frequency configuration done here
void PCG_A_B_Config()
{
	*pPCG_CTLA1 = CLKA_DIVIDER | 0x800000; //--To allign FS rising with CLK falling from PCG for I2S mode
	*pPCG_CTLA0 = FSA_DIVIDER | ENFSA | ENCLKA ; 
	
	*pPCG_CTLB1 = CLKB_DIVIDER; 
	*pPCG_CTLB0 = FSA_DIVIDER | ENFSB | ENCLKB ;

	*pPCG_PW = 0;
	*pPCG_SYNC = 0;	
}
//-------------------------------------------------------------


void SPDIF_RX_Init()
{
	//SPDIF_RX control register configuration done here if needed.
}
